(define-module (ui rd_url ajax)
               :use-module (alterator ajax)
               :use-module (alterator woo)
               :use-module (alterator card-index)
               :export (init))

; Debug string (paste in for your needs).
;(format #t "XXX: ~S~%" rdp_profile_name)

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; CREATE PAGE
;; ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

; Create profile form
(define (create-new-profile)
  (catch/message
    (lambda()
      (hide-all)
      ; Show form
      (form-update-visibility "new_profile_invalid" #f)
      (form-update-visibility '("new_profile_area") #t)
      ; Reset protocol name
      (form-update-value "new_profile_name"  ""))))


; Create new profile based on entered values
(define (create-new-profile-confirm)
  (let
    ((name (form-value "new_profile_name")))
    ; Test for correctness input	
    (if (regexp-exec (make-regexp (string-append "^" "" "$") regexp/extended) name)
      ; Empty name
      (and
        (form-update-value "new_profile_invalid_reason" "Profile name can't be empty")
        (form-update-visibility "new_profile_invalid" #t))
      ; Good name
      (and (catch/message
             (lambda () (woo-new "/rd"
                                 'name name
                                 'protocol "url")))
           (form-update-visibility '("new_profile_area") #f)
           (select-profile name)))))

; Select just created profile
(define (select-profile name)
  (form-update-value "avail_profiles" name)
  (edit-profile))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; URL
;; ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

; URL init
(define (url-init)
  (form-bind "url_btn_reset" "click" edit-profile)
  (form-bind "url_btn_save"  "click" url-write))

; Save URL profile
(define (url-write)
  (catch/message (lambda()
                   (apply woo "url" "/rd_url/write"
                          (form-value-list url-parameters))))
  (edit-profile))

; URL labels
(define url-labels '(
                     "url_profile_name"
                     ))

; URL inputs
(define url-inputs '(
                     "url_urlpath"
                     ))

; Known URL parameter
(define url-parameters (append '("avail_profiles")
                               (append url-labels
                                       url-inputs
                                       )))

; Fill wigets with default parameters
(define (set-default-parameters)
  (form-update-value "url_urlpath" "http://ya.ru")
  )

; Edit URL
(define (url-edit url_profile_name)

	(and

	; Reset all values
	(for-each (lambda(p) (form-update-value p "")) url-parameters)

	; Select profile in profiles list
	(form-update-value "avail_profiles" url_profile_name)

	; Set default parameters
	(set-default-parameters)

	; Read current settings
	(let* ((profile_info (woo "url" "/rd_url/read" 'url_profile_name url_profile_name 'language (form-value "language"))))
		(form-update-value-list url-parameters profile_info))

	; Turn form visibility
	(form-update-visibility '("url_area") #t)))


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; MAIN MODULE
;; ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

; Confirm delete selected profile
(define (delete-profile-confirm)
  (hide-all)
  (let ((profile_name (f-get-ci-value "avail_profiles")))
    (form-update-value "delete_profile_name" profile_name)
    ; Show form
    (form-update-visibility '("delete_profile_area") #t)))
           

; Delete profile
(define (delete-profile)
  (let ((profile_name (f-get-ci-value "avail_profiles")))
    (and 
      (catch/message (lambda() (woo-delete "/rd" 'profile_name profile_name 'language (form-value "language"))))
      (init-interface))))

; Interface initialization
(define (init-interface) 
  (catch/message 
    (lambda()
      ; Hide all modules
      (hide-all)
      ; Ask for available profiles
      (form-update-enum "avail_profiles" (woo-list "/rd_url/avail_profiles" 'language (form-value "language"))))))

; Hide all
(define (hide-all)
  (form-update-visibility '("new_profile_area" "delete_profile_area" "delete_button" "url_area") #f))

; Edit profile with specific type
(define (edit-profile)
  ; Update list, keep previous value or set the first one, return value
  (f-update-ci-list "avail_profiles" "/rd_url/avail_profiles")
  ; Get profile info
  (let* ((profile_name (f-get-ci-value "avail_profiles")))
    (hide-all)
    (form-update-visibility '("delete_button") #t)
    (url-edit profile_name)))

; Initialization routine
(define (init)
  ; Interconnect button with action 
  (form-bind "button_create_new_profile" "click" create-new-profile)
  (form-bind "button_create_new_profile_confirm" "click" create-new-profile-confirm)
  (form-bind "delete_button" "click" delete-profile-confirm)
  (form-bind "delete_profile_confirm" "click" delete-profile)
  (form-bind "delete_profile_cancel" "click" edit-profile)
  ; Selection profile lead to it editing
  (form-bind "avail_profiles" "change" edit-profile)
  ; Interface initialization
  (init-interface)
  ; Init modules
  (url-init))
